﻿using Infragistics.Controls.Grids;
using Infragistics.Olap;
using Infragistics.Olap.FlatData;
using System;
using System.Collections;
using System.Collections.Generic;
using System.Linq;
using System.Linq.Expressions;
using System.Reflection;
using System.Reflection.Emit;
using System.Text;
using System.Threading;
using System.Threading.Tasks;
using System.Windows;
using System.Windows.Controls;
using System.Windows.Data;
using System.Windows.Documents;
using System.Windows.Input;
using System.Windows.Media;
using System.Windows.Media.Imaging;
using System.Windows.Navigation;
using System.Windows.Shapes;

namespace DynamicTypeTest
{
    /// <summary>
    /// Interaction logic for MainWindow.xaml
    /// </summary>
    public partial class MainWindow : Window
    {
        
        public MainWindow()
        {
            InitializeComponent();

            DynamicTypeBuilder dtb = new DynamicTypeBuilder()
            {
                DynamicAssemblyName = "MyAssembly",
                DynamicTypeName = "MyType"
            };

            DynamicTypePropertyInfo propInfo1 = new DynamicTypePropertyInfo()
            {
                PropertyName = "Date123",
                PropertyType = typeof(DateTime),
                HasSetter = true
            };
            
            DynamicTypePropertyInfo propInfo2 = new DynamicTypePropertyInfo()
            {
                PropertyType = typeof(string),
                PropertyName = "City123",
                HasSetter = true
            };
            
            List<DynamicTypePropertyInfo> dtpi = new List<DynamicTypePropertyInfo>();
            dtpi.Add(propInfo1);
            dtpi.Add(propInfo2);
             

            string[] cities = "Sofia;London;New York;Berlin;Seattle;Mellvile;Tokyo;Shanghai;San Francisco;Trenton;Camden;".Split(';').ToArray();


            Type dynamicType = dtb.GenerateType(dtpi);

            Type listType = typeof(List<>);

            Type genericListType = listType.MakeGenericType(dynamicType);

            IList list = (IList)Activator.CreateInstance(genericListType);

            Random r = new Random();

            for (int i = 0; i < 10; i++)
            {
                dynamic d = Activator.CreateInstance(dynamicType);

                PropertyInfo pi1 = dynamicType.GetProperty("Date123");
                pi1.SetValue(d, DateTime.Today.AddYears(r.Next(0, 20)).AddDays(r.Next(1,10)));

                PropertyInfo pi2 = dynamicType.GetProperty("City123");
                pi2.SetValue(d, cities[i]);

                list.Add(d);
            }


            FlatDataSource FDS = new FlatDataSource() { ItemsSource = list, PreserveMembersOrder = false };

            HierarchyDescriptor hd = new HierarchyDescriptor() { SourcePropertyName = "Date123" };

            hd.LevelDescriptors.Add(new HierarchyLevelDescriptor() { LevelName = "All Periods" });
            hd.LevelDescriptors.Add(new HierarchyLevelDescriptor() { LevelName = "Years", LevelExpressionPath="Date123"  });

            Expression<Func<object, int>> expression = (e) => MyFunc(e);     

            hd.LevelDescriptors[1].OrderByKeyExpression = expression;

            FDS.HierarchyDescriptors.Add(hd);

            pivotGrid.DataSource = FDS;
            dataSelector.DataSource = FDS;
        }        

        public static int MyFunc(object e) 
        {
            Type t = e.GetType();
            PropertyInfo pi = t.GetProperty("Date123");

            var DT = pi.GetValue(e);

            DateTime d = ((DateTime)DT);
           
           return d.Year;
        }

    }



    


}
